"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "draftMode", {
    enumerable: true,
    get: function() {
        return draftMode;
    }
});
const _requestasyncstorageexternal = require("../../client/components/request-async-storage.external");
const _staticgenerationasyncstorageexternal = require("../../client/components/static-generation-async-storage.external");
const _dynamicrendering = require("../app-render/dynamic-rendering");
function draftMode() {
    const callingExpression = 'draftMode';
    const requestStore = (0, _requestasyncstorageexternal.getExpectedRequestStore)(callingExpression);
    const staticGenerationStore = _staticgenerationasyncstorageexternal.staticGenerationAsyncStorage.getStore();
    if (process.env.NODE_ENV === 'development' && !(staticGenerationStore == null ? void 0 : staticGenerationStore.isPrefetchRequest)) {
        const route = staticGenerationStore == null ? void 0 : staticGenerationStore.route;
        return createExoticDraftModeWithDevWarnings(requestStore.draftMode, route);
    } else {
        return createExoticDraftMode(requestStore.draftMode);
    }
}
const CachedDraftModes = new WeakMap();
function createExoticDraftMode(underlyingProvider) {
    const cachedDraftMode = CachedDraftModes.get(underlyingProvider);
    if (cachedDraftMode) {
        return cachedDraftMode;
    }
    const instance = new DraftMode(underlyingProvider);
    const promise = Promise.resolve(instance);
    CachedDraftModes.set(underlyingProvider, promise);
    Object.defineProperty(promise, 'isEnabled', {
        get () {
            return instance.isEnabled;
        },
        set (newValue) {
            Object.defineProperty(promise, 'isEnabled', {
                value: newValue,
                writable: true,
                enumerable: true
            });
        },
        enumerable: true,
        configurable: true
    });
    promise.enable = instance.enable.bind(instance);
    promise.disable = instance.disable.bind(instance);
    return promise;
}
function createExoticDraftModeWithDevWarnings(underlyingProvider, route) {
    const cachedDraftMode = CachedDraftModes.get(underlyingProvider);
    if (cachedDraftMode) {
        return cachedDraftMode;
    }
    const instance = new DraftMode(underlyingProvider);
    const promise = Promise.resolve(instance);
    CachedDraftModes.set(underlyingProvider, promise);
    Object.defineProperty(promise, 'isEnabled', {
        get () {
            const expression = 'draftMode().isEnabled';
            warnForSyncAccess(route, expression);
            return instance.isEnabled;
        },
        set (newValue) {
            Object.defineProperty(promise, 'isEnabled', {
                value: newValue,
                writable: true,
                enumerable: true
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(promise, 'enable', {
        value: function get() {
            const expression = 'draftMode().enable()';
            warnForSyncAccess(route, expression);
            return instance.enable.apply(instance, arguments);
        }
    });
    Object.defineProperty(promise, 'disable', {
        value: function get() {
            const expression = 'draftMode().disable()';
            warnForSyncAccess(route, expression);
            return instance.disable.apply(instance, arguments);
        }
    });
    return promise;
}
class DraftMode {
    constructor(provider){
        this._provider = provider;
    }
    get isEnabled() {
        return this._provider.isEnabled;
    }
    enable() {
        const store = _staticgenerationasyncstorageexternal.staticGenerationAsyncStorage.getStore();
        if (store) {
            // We we have a store we want to track dynamic data access to ensure we
            // don't statically generate routes that manipulate draft mode.
            (0, _dynamicrendering.trackDynamicDataAccessed)(store, 'draftMode().enable()');
        }
        return this._provider.enable();
    }
    disable() {
        const store = _staticgenerationasyncstorageexternal.staticGenerationAsyncStorage.getStore();
        if (store) {
            // We we have a store we want to track dynamic data access to ensure we
            // don't statically generate routes that manipulate draft mode.
            (0, _dynamicrendering.trackDynamicDataAccessed)(store, 'draftMode().disable()');
        }
        return this._provider.disable();
    }
}
function warnForSyncAccess(route, expression) {
    const prefix = route ? ` In route ${route} a ` : 'A ';
    console.error(`${prefix}\`draftMode()\` property was accessed directly with \`${expression}\`. \`draftMode()\` now returns a Promise and the return value should be awaited before accessing properties of the underlying draftMode object. In this version of Next.js direct access to \`${expression}\` is still supported to facilitate migration but in a future version you will be required to await the result. If this \`draftMode()\` use is inside an async function await the return value before accessing attempting iteration. If this use is inside a synchronous function then convert the function to async or await the call from outside this function and pass the result in.`);
}

//# sourceMappingURL=draft-mode.js.map